@@ -1,9 +1,10 @@
 # Standard Library
 import json
+import os
 import time
 from functools import lru_cache
 from pprint import pprint as pp
-from typing import List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 # Third Party
 import invoke
@@ -15,15 +16,60 @@ POLL_DELAY = 5
 
 
 @lru_cache(maxsize=None)
-def default_dbfs_artifact_path() -> str:
+def default_dbfs_artifact_path(branch_name: Optional[str] = None) -> str:
     """Helper to construct a default artifact path to upload to in DBFS."""
-    return f"dbfs:/FileStore/wheels/{poetry_project_name()}/{git_current_branch()}/"
+    if branch_name is None:
+        branch_name = git_current_branch()
+    return f"dbfs:/FileStore/wheels/{poetry_project_name()}/{branch_name}/"
 
 
 @lru_cache(maxsize=None)
-def default_dbfs_wheel_path() -> str:
+def default_dbfs_wheel_path(branch_name: Optional[str] = None) -> str:
     """Helper to construct a default artifact path to upload to in DBFS."""
-    return f"{default_dbfs_artifact_path()}{poetry_wheelname()}"
+    return f"{default_dbfs_artifact_path(branch_name)}{poetry_wheelname()}"
+
+
+def list_jobs(profile: Optional[str] = None) -> Dict[str, str]:
+    """List Jobs from databricks workspace.
+
+    Returns a dictionary with job names as keys and job IDs as values.
+    """
+    profile_flag = f"--profile {profile}" if profile else ""
+    result = invoke.run(f"databricks {profile_flag} jobs list --output JSON", hide=True)
+    run_status = json.loads(result.stdout)
+    jobs = {e["settings"]["name"]: e["job_id"] for e in run_status["jobs"]}
+    return jobs
+
+
+def create_or_reset_job(
+    json_payload: Dict[str, Any], profile: Optional[str] = None, job_id: Optional[str] = None
+) -> Any:
+    """Create or Reset a Databricks Job definition.
+
+    - Takes a parsed JSON payload dictionary,
+    - Encodes it into a temporary JSON file
+    - Runs the `create` or `reset` command depending on the presence of `job_id`
+    - cleans up temporary file
+    - returns response
+
+    Payload must comply with:
+    https://docs.databricks.com/dev-tools/api/latest/jobs.html#
+    """
+    profile_flag = f"--profile {profile}" if profile else ""
+    json_filename = "temp_job_file.json"
+    if os.path.exists(json_filename):
+        os.remove(json_filename)
+
+    with open(json_filename, "w") as f:
+        json.dump(json_payload, f)
+
+    if job_id:
+        result = invoke.run(f"databricks {profile_flag} jobs reset --job-id {job_id} --json-file {json_filename}")
+    else:
+        result = invoke.run(f"databricks {profile_flag} jobs create --json-file {json_filename}")
+
+    os.remove(json_filename)
+    return result.stdout
 
 
 def wait_for_cluster_status(
@@ -31,14 +77,17 @@ def wait_for_cluster_status(
     profile: Optional[str],
     cluster_id: Optional[str],
     target_status: List[str] = ["RUNNING"],
-    failure_status: List[str] = [],
+    failure_status: List[str] = [
+        "ERROR",
+        "UNKNOWN",
+    ],  # https://docs.databricks.com/dev-tools/api/latest/clusters.html#clusterstate
 ) -> None:
     """Poll cluster status until in desired state."""
     current_status = None
+    profile_flag = f"--profile {profile}" if profile else ""
+
     while current_status not in target_status:
-        result = c.run(
-            f"databricks --profile {profile} clusters events --cluster-id {cluster_id} --output JSON", hide=True
-        )
+        result = c.run(f"databricks {profile_flag} clusters events --cluster-id {cluster_id} --output JSON", hide=True)
         latest_event = json.loads(result.stdout)["events"][0]
         pp(latest_event["type"])
         current_status = latest_event["type"]
@@ -54,12 +103,18 @@ def wait_for_library_status(
     cluster_id: Optional[str],
     wheel: Optional[str],
     target_status: List[str] = ["INSTALLED"],
-    failure_status: List[str] = [],
+    failure_status: List[str] = [
+        "UNINSTALL_ON_RESTART",
+        "FAILED",
+        "SKIPPED",
+    ],  # https://docs.databricks.com/dev-tools/api/latest/libraries.html#libraryinstallstatus
 ) -> None:
     """Poll library statuses until target library in desired state."""
+    profile_flag = f"--profile {profile}" if profile else ""
     current_status = None
+
     while current_status not in target_status:
-        result = c.run(f"databricks --profile {profile} libraries cluster-status --cluster-id {cluster_id}", hide=True)
+        result = c.run(f"databricks {profile_flag} libraries cluster-status --cluster-id {cluster_id}", hide=True)
         statuses = json.loads(result.stdout)["library_statuses"]
         filtered_status = [
             status for status in statuses if "whl" in status["library"] and status["library"]["whl"] == wheel
@@ -77,14 +132,16 @@ def wait_for_run_status(
     profile: Optional[str],
     run_id: Optional[str],
     target_status: List[str] = ["TERMINATED"],
-    failure_status: List[str] = ["INTERNAL_ERROR"],
+    failure_status: List[str] = ["INTERNAL_ERROR", "SKIPPED"],
 ) -> None:
     """Poll run status until in desired state."""
+    profile_flag = f"--profile {profile}" if profile else ""
     current_status = None
     url = None
     run_status = {}
+
     while current_status not in target_status:
-        result = c.run(f"databricks --profile {profile} runs get --run-id {run_id}", hide=True)
+        result = c.run(f"databricks {profile_flag} runs get --run-id {run_id}", hide=True)
         run_status = json.loads(result.stdout)
         if url is None:
             url = run_status["run_page_url"]
@@ -99,7 +156,7 @@ def wait_for_run_status(
 
     pp(run_status["state"])
     print(url)
-    result = c.run(f"databricks --profile {profile} runs get-output --run-id {run_id}", hide=True)
+    result = c.run(f"databricks {profile_flag} runs get-output --run-id {run_id}", hide=True)
     output = json.loads(result.stdout)
 
     for k in ["error", "error_trace", "logs"]:
